#
#
# onestepmodules.pl
#

use Getopt::Long;
use File::Basename;
use Config;
use strict;
use warnings;

use PPM;

$PPM::VERSION = "2.1.6";



my ($BUGZILLA_INSTALL_F,$TARGETDRIVE,$BUGZILLA_FILES,$batch);

# get values for some parameters from the command line
    $BUGZILLA_INSTALL_F = $ARGV[0];
    $TARGETDRIVE = $ARGV[1];
    $BUGZILLA_FILES = $ARGV[2];

    $batch = "$BUGZILLA_INSTALL_F\\temp.bat";


print "Now executing 'onestepmodules' ...\n\n";



##############################################################################
# see if the database is ready
#
# Check if we have access to --MYSQL--
#
sub check_mysql()
{
    my ($startit);

    # see if the database is ready
    #
    # Check if we have access to --MYSQL--
    #
    # Is there an easy and quick way to see if mysql.exe is running?
    print "\n";
    print "\n";
    open(LIST, "$TARGETDRIVE/mysql/bin/mysqladmin -ubugs -pPleaseCharge ping | ");
    my ($textline);
    $textline = "";
    if (defined($textline = <LIST>))
    {
#       print LOGFILE $textline;
        print $textline;
    }
    else
    {
        $textline="";
    }
    close(LIST);

    if ($textline =~ / alive/)
    {
        print "mysql is alive\n";
        $startit = 0;
    }
    else
    {
        print "mysql is dead\n";
        $startit = 1;
    }



    return($startit);
}
##############################################################################






##############################################################################
#
# Check if we have access to the bugs database on mysql
#
# Some of this code is scammed from checksetup.pl
sub handle_mysql()
{

    my $my_db_host = "localhost";         # where is the database?
    my $my_db_port = 3306;                # which port to use
    my $my_db_name = "bugs";              # name of the MySQL database

    my $my_db_user = "bugs";              # user to attach to the MySQL database
    my $my_db_pass = 'PleaseCharge';


    # This settings are not yet changeable, because other code depends on
    # the fact that we use MySQL and not, say, PostgreSQL.
    my $db_base = 'mysql';

    my($dbh, $dsn);






use DBI;

    # one way to check and see if MYSQL is running is to execute the command
    # mysqladmin --user=bugs password ""
    # and see what text is output.  If it says stuff about "port", then
    # its not running -- well at least not on the default port.

    # Do we have the database itself?

    print "\n";
    print "\n";

    $dsn = "DBI:$db_base:;$my_db_host;$my_db_port";
    $dbh = DBI->connect($dsn, $my_db_user, $my_db_pass)
        or die "Can't connect to bugs table in myql database!\n";

    $dbh->disconnect if $dbh;
}
##############################################################################





##############################################################################
sub do_one($$)
{
    my $package = shift;
    my $location = shift;
    my $doit = 0;

    print "Verifying package '$package'...\n";

    #    , "location" => $location
    my $ok = VerifyPackage("package" => $package);
    # returns undef for error case
    if (! defined $ok)
    {
        print "Error verifying package '$package': $PPM::PPMERR\n";
        $doit = 1;
    }
    else
    {
        # 1=update needed
        $doit = $ok;
    }

    if ($doit == 1)
    {
        print "Installing package '$package'...\n";

        #    , "location" => $location
        if(!InstallPackage("package" => $package))
        {
            print "Error installing package '$package': $PPM::PPMERR\n";
        }
        else
        {
            # it would be nice to say what the version of the package is...
            print "Installed package '$package'\n";
        }
    }
    else
    {
        print "Update for '$package' not needed.\n";
    }

}
##############################################################################









##############################################################################
#
# This will install modules one-by-one
#
#
sub do_all_updates()
{
    # version 1.35
    #do_one("DBI", "http://www.cpan.org");
    #system "ppm install DBI";
    #423364 bytes
    do_one("DBI", "");

    #system "ppm install DBD-Mysql";
    #http://www.cpan.org/authors/id/J/JW/JWIED/DBD-mysql-2.1026.tar.gz
    do_one("DBD-Mysql", "");

    # version 1.14
    do_one("TimeDate", "");
    # "http://www.cpan.org/authors/id/G/GB/GBARR/TimeDate-1.14.tar.gz");
    # version 1.16 seems better

    #system "ppm install Chart";
    #system "ppm install Date-Calc";
    #system "ppm install Date-Manip";
    #system "ppm install GD";
    do_one("Chart", "");
    do_one("Date-Calc", "");
    do_one("Date-Manip", "");
    do_one("GD", "");



    # If this works, then you'll be very happy ...
    # ppm install bundle-bugzilla
#    do_one("bundle-bugzilla", "");


   # This is the only thing missing from perl-win32-bin-0.10.exe when using bugzilla 2.17.4
   do_one("$BUGZILLA_INSTALL_F\\TimeDateppd\\TimeDate.ppd",  "");
}
##############################################################################










##############################################################################
# All the ppm modules are installed.  This section configures and checks the
# system using the checksetup.pl that comes in the bugzilla tar.
{


    ###do_all_updates();
    if (! -f "$TARGETDRIVE\\perl\\site\\lib\\Date\\Format.pm")
    {
        do_one("$BUGZILLA_INSTALL_F\\TimeDateppd\\TimeDate.ppd",  "");
    }

#    do_one("DBI", "");
#    do_one("DBD-Mysql", "");


    # run "perl checksetup.pl" the first time in order
    # to create 'localconfig' file
    if (! -f "$TARGETDRIVE\\$BUGZILLA_FILES\\localconfig")
    {
        print "Info: interpret checksetup.pl\n";
        system "$BUGZILLA_INSTALL_F\\checksetup.bat";
    }


    # see if the database is ready
    #
    # Check if we have access to --MYSQL--
    #
    my ($startit);

    $startit = check_mysql();
    if ($startit)
    {
        print "\n\nTrying to start mysql ...\n";
        system("start \"mysql\" \"$BUGZILLA_INSTALL_F\\start_mysql.bat\"");
        print "\n";

        # wait xx seconds
        sleep(10.0);
    }
    $startit = check_mysql();
    die "Can't get mysql going\n" if ($startit);




    # note that re-running this should not be a problem.
    if ( -f "$TARGETDRIVE\\$BUGZILLA_FILES\\localconfig")
    {
        # need a password in c:\bugzilla\localconfig
        # The localconfig file only exists after the first execution of checksetup
        # so this code needs to be between the two calls.
        # Should this script prompt for the password and use the answer?
        # change $TARGETDRIVE\\$BUGZILLA_FILES\\localconfig


        open BATCH, ">", "$batch";
        print BATCH "$TARGETDRIVE\n";
        print BATCH "cd \\$BUGZILLA_FILES\n";
        print BATCH "$TARGETDRIVE\\mysql\\bin\\replace.exe \"\$db_pass = \" \"\$db_pass='PleaseCharge' .\" -- $TARGETDRIVE\\$BUGZILLA_FILES\\localconfig\n";
#C:\mysql\bin\replace.exe "C:\\mysql\\bin\\mysql.exe" "C:/mysql/bin/mysql.exe" -- C:\bugzilla\localconfig
        print BATCH "$TARGETDRIVE\\mysql\\bin\\replace.exe \"$TARGETDRIVE\\\\mysql\\\\bin\\\\mysql.exe\" \"$TARGETDRIVE/mysql/bin/mysql.exe\" -- $TARGETDRIVE\\$BUGZILLA_FILES\\localconfig\n";

        close BATCH;

    #   print LOGFILE "\n---------------- start of patch transaction  ----------------\n";
        open(LIST, "$batch | ");    # or die "can't run perl to patch checksetup.pl";
        my ($textline);
        while (defined($textline = <LIST>))
        {
    #       print LOGFILE $textline;
            print $textline;
        }
        close(LIST);
    #   print LOGFILE "---------------- end of patch transaction  ----------------\n\n";




        # run "perl checksetup.pl" the second time in order to
        # set up the database tables and such
        print "Info: interpret checksetup.pl a second time\n";
    #    print LOGFILE "Info: interpret checksetup.pl a second time\n";
        system "$BUGZILLA_INSTALL_F\\checksetup.bat";





    }
    else
    {
       print "FATAL ERROR: bugzilla didn't pass the checksetup\n";
    }
}
##############################################################################

